@@ -1,6 +1,7 @@
 package Dancer::Plugin;
 use strict;
 use warnings;
+use Carp;
 
 use base 'Exporter';
 use Dancer::Config 'setting';
@@ -43,8 +44,14 @@ sub register($&) {
     my $plugin_name = caller();
 
     if (grep { $_ eq $keyword } @_reserved_keywords) {
-        die "You can't use $keyword, this is a reserved keyword";
+        croak "You can't use $keyword, this is a reserved keyword";
     }
+    while (my ($plugin, $keywords) = each %$_keywords) {
+        if (grep { $_->[0] eq $keyword } @$keywords) {
+            croak "You can't use $keyword, this is a keyword reserved by $plugin";
+        }
+    }
+
     $_keywords->{$plugin_name} ||= [];
     push @{$_keywords->{$plugin_name}}, [$keyword => $code];
 }
@@ -56,15 +63,16 @@ sub register_plugin {
     my @symbols = set_plugin_symbols($plugin);
     {
         no strict 'refs';
-        @{"${plugin}::ISA"} = ('Exporter', 'Dancer::Plugin');
-        @{"${plugin}::EXPORT"} = @symbols;
+        # tried to use unshift, but it yields an undef warning on $plugin (perl v5.12.1)
+        @{"${plugin}::ISA"} = ('Exporter', 'Dancer::Plugin', @{"${plugin}::ISA"});
+        push @{"${plugin}::EXPORT"}, @symbols;
     }
     return 1;
 }
 
 sub load_plugin {
     my ($plugin) = @_;
-    die "load_plugin is DEPRECATED, you must use 'use' instead";
+    croak "load_plugin is DEPRECATED, you must use 'use' instead";
 }
 
 sub set_plugin_symbols {
